@@ -17,11 +17,7 @@ from django.db import connections
 from django.db.utils import DEFAULT_DB_ALIAS
 from django.test.testcases import TransactionTestCase
 
-
-try:
-    from importlib import reload
-except ImportError:  # Python < 3
-    pass
+from importlib import reload
 import sys
 import unittest
 
@@ -33,12 +29,10 @@ from django.test import TestCase as DjangoTestCase
 from django.test.client import RequestFactory, Client
 from django.test.utils import override_settings
 
-from mock import patch
+from unittest.mock import patch
 
 from watchman import checks, views
 
-PYTHON_VERSION = sys.version_info[0]
-
 
 class AuthenticatedUser(AnonymousUser):
     @property
@@ -55,12 +49,11 @@ class AuthenticatedUser(AnonymousUser):
         return CallableTrue()
 
 
-if django.VERSION >= (1, 7):
-    # Initialize Django
-    django.setup()
+# Initialize Django
+django.setup()
 
-    # Silence MIDDLEWARE_CLASSES warning as this is not an actual Django project
-    settings.SILENCED_SYSTEM_CHECKS = ['1_7.W001']
+# Silence MIDDLEWARE_CLASSES warning as this is not an actual Django project
+settings.SILENCED_SYSTEM_CHECKS = ['1_7.W001']
 
 
 def reload_settings():
@@ -85,12 +78,8 @@ class TestWatchman(unittest.TestCase):
         request = RequestFactory().get('/')
         response = views.status(request)
 
-        if PYTHON_VERSION == 2:
-            content = json.loads(response.content)
-            self.assertItemsEqual(expected_checks, content.keys())
-        else:
-            content = json.loads(response.content.decode('utf-8'))
-            self.assertCountEqual(expected_checks, content.keys())
+        content = json.loads(response.content.decode('utf-8'))
+        self.assertCountEqual(expected_checks, content.keys())
 
     def test_check_database_handles_exception(self):
         response = checks._check_database('foo')
@@ -98,10 +87,7 @@ class TestWatchman(unittest.TestCase):
         self.assertEqual(response['foo']['error'], "The connection foo doesn't exist")
 
     def test_check_cache_handles_exception(self):
-        if django.VERSION < (1, 7):
-            expected_error = "Could not find backend 'foo': Could not find backend 'foo': foo doesn't look like a module path"
-        else:
-            expected_error = "Could not find config for 'foo' in settings.CACHES"
+        expected_error = "Could not find config for 'foo' in settings.CACHES"
 
         response = checks._check_cache('foo')
         self.assertFalse(response['foo']['ok'])
@@ -114,12 +100,8 @@ class TestWatchman(unittest.TestCase):
         })
         response = views.status(request)
 
-        if PYTHON_VERSION == 2:
-            content = json.loads(response.content)
-            self.assertItemsEqual(expected_checks, content.keys())
-        else:
-            content = json.loads(response.content.decode('utf-8'))
-            self.assertCountEqual(expected_checks, content.keys())
+        content = json.loads(response.content.decode('utf-8'))
+        self.assertCountEqual(expected_checks, content.keys())
 
     def test_response_is_404_for_checked_and_skipped_check(self):
         # This is a bit of a weird one, basically if you explicitly include and
@@ -141,12 +123,8 @@ class TestWatchman(unittest.TestCase):
         response = views.status(request)
         self.assertEqual(response.status_code, 200)
 
-        if PYTHON_VERSION == 2:
-            content = json.loads(response.content)
-            self.assertItemsEqual({'databases': []}, content)
-        else:
-            content = json.loads(response.content.decode('utf-8'))
-            self.assertCountEqual({'databases': []}, content)
+        content = json.loads(response.content.decode('utf-8'))
+        self.assertCountEqual({'databases': []}, content)
 
     def test_response_404_when_none_specified(self):
         request = RequestFactory().get('/', data={
@@ -155,12 +133,8 @@ class TestWatchman(unittest.TestCase):
         response = views.status(request)
         self.assertEqual(response.status_code, 404)
 
-        if PYTHON_VERSION == 2:
-            content = json.loads(response.content)
-            self.assertItemsEqual({'message': 'No checks found', 'error': 404}, content)
-        else:
-            content = json.loads(response.content.decode('utf-8'))
-            self.assertCountEqual({'message': 'No checks found', 'error': 404}, content)
+        content = json.loads(response.content.decode('utf-8'))
+        self.assertCountEqual({'message': 'No checks found', 'error': 404}, content)
 
     @override_settings(WATCHMAN_TOKEN='ABCDE')
     @override_settings(WATCHMAN_AUTH_DECORATOR='watchman.decorators.token_required')
@@ -176,6 +150,17 @@ class TestWatchman(unittest.TestCase):
 
         self.assertEqual(response.status_code, 200)
 
+    @override_settings(WATCHMAN_TOKEN='ABCDE')
+    def test_version_header_not_included_when_token_auth_fails(self):
+        # Have to manually reload settings here because override_settings
+        # happens after self.setUp(), but before self.tearDown()
+        reload_settings()
+        request = RequestFactory().get('/')
+
+        response = views.status(request)
+        self.assertEqual(response.status_code, 403)
+        self.assertFalse(response.has_header('X-Watchman-Version'))
+
     @override_settings(WATCHMAN_TOKEN='ABCDE')
     @override_settings(WATCHMAN_AUTH_DECORATOR='watchman.decorators.token_required')
     def test_login_not_required_with_authorization_header(self):
@@ -243,7 +228,16 @@ class TestWatchman(unittest.TestCase):
         response = views.status(request)
         self.assertEqual(response.status_code, 200)
 
+    def test_response_version_header_missing_by_default(self):
+        request = RequestFactory().get('/')
+        response = views.status(request)
+        self.assertFalse(response.has_header('X-Watchman-Version'))
+
+    @override_settings(EXPOSE_WATCHMAN_VERSION=True)
     def test_response_version_header(self):
+        # Have to manually reload settings here because override_settings
+        # happens after self.setUp()
+        reload_settings()
         request = RequestFactory().get('/')
         response = views.status(request)
         self.assertTrue(response.has_header('X-Watchman-Version'))
@@ -336,10 +330,83 @@ class TestWatchmanDashboard(unittest.TestCase):
         response = views.dashboard(request)
         self.assertEqual(response.status_code, 200)
 
-    def test_response_version_header(self):
+    def test_response_version_header_and_html_missing_by_default(self):
+        request = RequestFactory().get('/')
+        response = views.dashboard(request)
+        self.assertFalse(response.has_header('X-Watchman-Version'))
+        self.assertNotIn('Watchman version:', response.content.decode())
+
+    @override_settings(EXPOSE_WATCHMAN_VERSION=True)
+    def test_response_has_version_header_and_html(self):
+        # Have to manually reload settings here because override_settings
+        # happens after self.setUp()
+        reload_settings()
         request = RequestFactory().get('/')
         response = views.dashboard(request)
         self.assertTrue(response.has_header('X-Watchman-Version'))
+        self.assertIn('Watchman version:', response.content.decode())
+
+
+class TestPing(unittest.TestCase):
+    def setUp(self):
+        # Ensure that every test executes with separate settings
+        reload_settings()
+
+    def test_returns_pong(self):
+        request = RequestFactory().get('/')
+        response = views.ping(request)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.content.decode(), 'pong')
+        self.assertEqual(response['Content-Type'], 'text/plain')
+
+
+class TestBareStatus(unittest.TestCase):
+    def setUp(self):
+        # Ensure that every test executes with separate settings
+        reload_settings()
+
+    def test_bare_status_success(self):
+        request = RequestFactory().get('/')
+        response = views.bare_status(request)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.content.decode(), '')
+
+    @patch('watchman.checks._check_databases')
+    @override_settings(WATCHMAN_ERROR_CODE=503)
+    def test_bare_status_error(self, patched_check_databases):
+        reload_settings()
+        # Fake a DB error, ensure we get our error code
+        patched_check_databases.return_value = [{
+            "foo": {
+                "ok": False,
+                "error": "Fake DB Error",
+                "stacktrace": "Fake DB Stack Trace",
+            },
+        }]
+        request = RequestFactory().get('/', data={
+            'check': 'watchman.checks.databases',
+        })
+        response = views.bare_status(request)
+        self.assertEqual(response.status_code, 503)
+        self.assertEqual(response.content.decode(), '')
+
+    @patch('watchman.checks._check_databases')
+    def test_bare_status_default_error(self, patched_check_databases):
+        reload_settings()
+        # Fake a DB error, ensure we get our error code
+        patched_check_databases.return_value = [{
+            "foo": {
+                "ok": False,
+                "error": "Fake DB Error",
+                "stacktrace": "Fake DB Stack Trace",
+            },
+        }]
+        request = RequestFactory().get('/', data={
+            'check': 'watchman.checks.databases',
+        })
+        response = views.bare_status(request)
+        self.assertEqual(response.status_code, 500)
+        self.assertEqual(response.content.decode(), '')
 
 
 class TestEmailCheck(DjangoTestCase):
@@ -413,3 +480,27 @@ class TestEmailCheck(DjangoTestCase):
             'foo': 'bar',
         }
         self.assertEqual(sent_email.extra_headers, expected_headers)
+
+    def def_test_email_with_default_sender(self):
+        checks._check_email()
+
+        # Test that one message has been sent.
+        self.assertEqual(len(mail.outbox), 1)
+
+        sent_email = mail.outbox[0]
+        expected_sender = 'watchman@example.com'
+        self.assertEqual(sent_email.from_email, expected_sender)
+
+    @override_settings(WATCHMAN_EMAIL_SENDER='custom@example.com')
+    def def_test_email_with_custom_sender(self):
+        # Have to manually reload settings here because override_settings
+        # happens after self.setUp()
+        reload_settings()
+        checks._check_email()
+
+        # Test that one message has been sent.
+        self.assertEqual(len(mail.outbox), 1)
+
+        sent_email = mail.outbox[0]
+        expected_sender = 'custom@example.com'
+        self.assertEqual(sent_email.from_email, expected_sender)
